<cfimport prefix="l" taglib="layout/" />

<cfoutput>
<l:header title="Server-Side Validation">
	<script type="text/javascript">
		// <![CDATA[
		function validateForm() {
			var formIsClean = true;
			formIsClean = validateName("name") && formIsClean;
			formIsClean = validateUsername("username") && formIsClean;
			formIsClean = validateEmail("email") && formIsClean;
			return formIsClean;
		}
		function validateName(el) {
			clearError(el);
			if ($F(el) == "") {
				setError(el, "You must provide your name");
				return false;
			}
			return true;
		}
		function validateUsername(el) {
			clearError(el);
			if ($F(el).length < 4) {
				setError(el, "Usernames must be at least 4 characters long");
				return false;
			}
			if ($F(el).length > 16) {
				setError(el, "Usernames cannot be more than 16 characters long");
				return false;
			}
			 
			new Ajax.Request("is_username_valid_action.cfm", {
				postBody: {
					username: $F(el)
				},
				onComplete: function(t) {
					if (isValid = t.responseText != "true") {
						setError(el, "The username you've chosen is not unique; please select another");
					}
				}
			});

			return true;
		}
		function validateEmail(el) {
			clearError(el);
			if (! /.+@.+/.test($F(el))) {
				setError(el, "You must provide a valid email address");
				return false;
			}
			return true;
		}
		function clearError(el) {
			setError(el, "");
		}
		function setError(el, message) {
			$("message_" + $(el).id).innerHTML = message;
		}
		// ]]>
	</script>
</l:header>
<form onsubmit="return validateForm();">
<p><strong>Problem:</strong> We can't do all form validation without server-side access.
</p>
<p><strong>Example:</strong> We can check username length requirements, but not uniqueness.
</p>
<p><strong>Solution:</strong> Combine client-side validation with server-side data access
to effectively perform server-side without your visitor being aware you're doing more than
client-side validation.
</p>
<p><strong>Caveats:</strong> Packaging is crucial, because you're effectively doing form
validation THREE times (client, client/server, and server), rather than the usual one
or two.
</p>
<div class="clearAll"></div>
<div class="formItem">
	<label for="name">Name:</label>
	<div class="indicator" id="indicator_name">*</div>
	<div class="formField">
		<input name="name" id="name" onchange="validateName(this);" />
		<span class="message" id="message_name"></span>
	</div>
</div>
<div class="formItem">
	<label for="username">Username:</label>
	<div class="indicator" id="indicator_username">*</div>
	<div class="formField">
		<input name="username" id="username" onchange="validateUsername(this);" />
		<span class="message" id="message_username"></span>
	</div>
</div>
<div class="formItem">
	<label for="email">Email:</label>
	<div class="indicator" id="indicator_email">*</div>
	<div class="formField">
		<input name="email" id="email" onchange="validateEmail(this.id);" />
		<span class="message" id="message_email"></span>
	</div>
</div>
<div class="formItem">
	<label></label>
	<div class="indicator"></div>
	<div class="formField">
		<input type="submit" value="Submit" />
	</div>
</div>
<div class="clearAll"></div>
</form>
<l:footer />
</cfoutput>